function [signalRecon] = subsampleSpectrum(signal,M,scenario)
% [signalRecon] = subsampleSpectrum(signal,M,scenario)
%
% Reconstruct a signal from undersampled frequency spectrum.
%
% Input:
%   signal = original time-domain signal
%   M = number of frequency-domain samples drawn randomly
%   scenario = 1: random sampling without replacement
%   scenario = 2: random sampling with replacement
%
% Output: 
%   signalRecon = reconstructed signal in time domain based on
%                 incomplete frequency-domain samples
%
% Random Spectral Sampling Toolbox - Roel Snieder and Michael B. Wakin
% Version 1.0 - March 23, 2022
% See README.txt


% Take the DFT of the original signal
N = length(signal);
idealSpectrum = fft(signal);

% Define the frequency-domain locations to keep spectrum samples
switch scenario
    case 1
        % uniform random sampling without replacement
        sampleProbabilities = (1/N)*ones(N,1);
        sampleLocs = datasample(1:N,M,'Weights',sampleProbabilities,'Replace',false);
    case 2
        % uniform random sampling with replacement
        sampleProbabilities = (1/N)*ones(N,1);
        sampleLocs = datasample(1:N,M,'Weights',sampleProbabilities,'Replace',true);
end
spectralSamples = idealSpectrum(sampleLocs);

% If sampling with replacement, some samples may be "kept" more than once. 
% Define subsampled spectrum with each sample scaled by its multiplicity 
% in spectralSamples.
subsampledSpectrum = zeros(N,1);
for ii = 1:M
    subsampledSpectrum(sampleLocs(ii)) = subsampledSpectrum(sampleLocs(ii)) + spectralSamples(ii);
end

% Reconstruct from the spectral samples
signalRecon = (N/M)*ifft(subsampledSpectrum);
